package org.intellij.openapi.editor;

import com.intellij.openapi.editor.Document;
import com.intellij.openapi.util.TextRange;

import org.intellij.openapi.util.OffsetRange;
import org.intellij.openapi.util.ReadOnlyCharArray;

public final class DocumentWrapper extends DocumentDelegate {

    public DocumentWrapper(Document document) {
        super(document);
    }

    public ReadOnlyCharArray getReadOnlyCharArray() {
        return new ReadOnlyCharArray(getChars(), getTextLength());
    }

    public String getLineText(int lineNumber) {
        int startOffset = getLineStartOffset(lineNumber);
        int count = getLineEndOffset(lineNumber) - startOffset;
        return getReadOnlyCharArray().newString(startOffset, count);
    }

    private int getLineEndOffsetIncluding(int lineNumber) {
        return lineNumber < getLineCount() - 1 ? getLineEndOffset(lineNumber) + 1 : getTextLength();
    }

    public void deleteLine(int lineNumber) {
        deleteString(getLineStartOffset(lineNumber), getLineEndOffsetIncluding(lineNumber));
    }

    boolean isLineStart(int offset) {
        return offset == getLineStartOffset(getLineNumber(offset));
    }

    public boolean isEmpty() {
        return getTextLength() == 0;
    }

    public int getLineStartOffset(int lineNumber) {
        return isEmpty() ? 0 : super.getLineStartOffset(lineNumber);
    }

    public int getLineEndOffset(int lineNumber) {
        return isEmpty() ? 0 : super.getLineEndOffset(lineNumber);
    }

    boolean isBlankLine(int lineNumber) {
        return "".equals(getLineText(lineNumber).trim());
    }

    public int firstBlackLineNumber(int fromLineNumber) {
        int blackLineNumber = -1;
        for (int lineNumber = fromLineNumber; lineNumber < getLineCount(); lineNumber++) {
            if (!isBlankLine(lineNumber)) {
                blackLineNumber = lineNumber;
                break;
            }
        }
        return blackLineNumber;
    }

    public int lastBlackLineNumber(int fromLineNumber) {
        int blackLineNumber = -1;
        for (int lineNumber = fromLineNumber; lineNumber >= 0; lineNumber--) {
            if (!isBlankLine(lineNumber)) {
                blackLineNumber = lineNumber;
                break;
            }
        }
        return blackLineNumber;
    }

    private char charAt(int offset) {
        return getChars()[offset];
    }

    public int getFirstBlackCharOffset(int lineNumber) {
        return firstBlackCharOnLineOffset(getLineStartOffset(lineNumber));
    }

    private int firstBlackCharOnLineOffset(int offset) {
        int lineEndOffset = getLineEndOffset(getLineNumber(offset));
        int resultOffset;
        for (resultOffset = offset; resultOffset < lineEndOffset; resultOffset++) {
            if (!Character.isWhitespace(charAt(resultOffset))) {
                break;
            }
        }
        return resultOffset;
    }

    private int lastBlackCharOnLineOffset(int offset) {
        int lineStartOffset = getLineStartOffset(getLineNumber(offset));
        int resultOffset;
        for (resultOffset = offset; resultOffset >= lineStartOffset; resultOffset--) {
            if (!Character.isWhitespace(charAt(resultOffset))) {
                break;
            }
        }
        return resultOffset;
    }

    public OffsetRange getAllSelection() {
        return new OffsetRange(0, getTextLength());
    }

    public TextRange getExtendedRange(TextRange textRange) {
        int endOffset = textRange.getEndOffset();
        {
            int lineNumber = getLineNumber(endOffset);
            endOffset = firstBlackCharOnLineOffset(endOffset);
            if (endOffset == getLineEndOffset(lineNumber)) {
                endOffset = getLineStartOffset(firstBlackLineNumber(lineNumber + 1));
            }
        }
        int startOffset = lastBlackCharOnLineOffset(textRange.getStartOffset() - 1) + 1;
        return new TextRange(startOffset, endOffset);
    }

}
